@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli-getaddrinfo.c,v 1.3 2004/06/03 16:54:49 evertonm Exp $
+  $Id: ruli-getaddrinfo.c,v 1.4 2004/06/05 19:42:29 evertonm Exp $
  */
 
 
@@ -218,7 +218,7 @@ static void solve(const char *fullname)
       }
     } /* show addresses */
 
-    freeaddrinfo(ai_res);
+    ruli_freeaddrinfo(ai_res);
 
   } /* submit query */
 
