@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: srvsearch.c,v 1.6 2004/05/21 18:23:50 evertonm Exp $
+  $Id: srvsearch.c,v 1.7 2004/06/03 16:54:49 evertonm Exp $
  */
 
 
@@ -194,6 +194,14 @@ static void do_query(ruli_res_t *res_ctx, const char *domain)
     const char *past_end   = domain + domain_len;
     const char *i = domain;
 
+    if (*i != '_') {
+      fprintf(stderr, 
+	      "%s: do_query(): could not match _service\n",
+	      prog_name);
+      
+      return;
+    }
+
     for (; i < past_end; ++i) {
       if (*i == '.') {
 	++i;
