@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_addr.c,v 1.6 2004/06/01 18:37:15 evertonm Exp $
+  $Id: ruli_addr.c,v 1.7 2004/06/06 04:27:48 evertonm Exp $
   */
 
 
@@ -300,7 +300,7 @@ int ruli_addr_print(FILE *out, const ruli_addr_t *addr)
 	  if (!(j % 2))
 	    wr += fprintf(out, ":");
 
-	wr += fprintf(out, "%02X", *i);
+	wr += fprintf(out, "%02X", (unsigned char) *i);
       }
 
       return wr;
@@ -346,7 +346,7 @@ int ruli_addr_snprint(char *buf, size_t size, const ruli_addr_t *addr)
 	    assert(wr < size);
 	  }
 
-	w = snprintf(buf + wr, size - wr, "%02X", *i);
+	w = snprintf(buf + wr, size - wr, "%02X", (unsigned char) *i);
 	if (w < 0)
 	  return -1;
 	wr += w;
