@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_fsm.c,v 1.21 2004/05/31 05:57:40 evertonm Exp $
+  $Id: ruli_fsm.c,v 1.22 2004/06/03 17:34:27 evertonm Exp $
 
 
   TIMEOUT NOTES:
@@ -35,7 +35,6 @@ Boston, MA 02111-1307, USA.
 #include <stdio.h>      /* FIXME: remove me [used for fprintf() debug] */
 #include <errno.h>      /* FIXME: remove me [used for strerror() debug] */
 #include <string.h>     /* FIXME: remove me [used for strerror() debug] */
-#include <sys/socket.h> /* FIXME: remove me [used for inet_ntoa() debug] */
 
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -636,9 +635,9 @@ static void query_want_connect_tcp(ruli_res_query_t *qry)
     server = ruli_res_get_curr_server(qry);
 
 #ifdef RULI_RES_DEBUG
-    fprintf(stderr, 
-	    "DEBUG: query_want_connect_tcp(): trying %s:%d\n", 
-	    inet_ntoa(server_addr->addr.ipv4), server_port);
+    fprintf(stderr, "DEBUG: query_want_connect_tcp(): trying ");
+    ruli_addr_print(stderr, server_addr);
+    fprintf(stderr, ":%d\n", server_port);
 #endif
 
     assert(qry->status == RULI_QRY_STAT_VOID);
