@@ -16,7 +16,7 @@
  */
 
 /*
-  $Id: ruli_getaddrinfo.c,v 1.4 2004/06/03 16:54:49 evertonm Exp $
+  $Id: ruli_getaddrinfo.c,v 1.6 2004/06/05 19:42:30 evertonm Exp $
 */
 
 
@@ -26,21 +26,63 @@
 #include <netinet/in.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <errno.h>
+
+#ifndef __USE_GNU
+#define __USE_GNU
+#endif
+
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE
+#endif
+
+#ifndef __USE_POSIX
+#define __USE_POSIX
+#endif
+
+#include <netdb.h>
+#include <stdio.h>
+#include <dlfcn.h>
 
 #include <ruli_getaddrinfo.h>
 #include <ruli_sync.h>
 #include <ruli_txt.h>
 #include <ruli_mem.h>
 
-#include <netdb.h>
-#include <stdio.h>
-#include <dlfcn.h>
+
+typedef struct {
+  struct addrinfo ai;
+  union {
+    struct sockaddr_in sa_inet;
+    struct sockaddr_in6 sa_inet6;
+  } sa;
+} _ruli_ai_addrbag;
 
 
 static int (*real_getaddrinfo)(const char *node, const char *service,
 			       const struct addrinfo *hints,
 			       struct addrinfo **res) = NULL;
 
+
+static int call_getaddrinfo(const char *node, const char *service,
+			    const struct addrinfo *hints,
+			    struct addrinfo **_res)
+{
+  if (!real_getaddrinfo)
+    real_getaddrinfo = (int (*)(const char *,
+				const char *,
+				const struct addrinfo *,
+				struct addrinfo **))
+      dlsym(RTLD_NEXT, "getaddrinfo");
+
+  if (!real_getaddrinfo) {
+    errno = ENOTSUP;
+    return EAI_SYSTEM;
+  }
+
+  return real_getaddrinfo(node, service, hints, _res);
+}
+
 int ruli_getaddrinfo(const char *node, const char *service,
 		     const struct addrinfo *hints, struct addrinfo **_res)
 {
@@ -50,48 +92,41 @@ int ruli_getaddrinfo(const char *node, const char *service,
   char *full_service;
   int srv_code, srv_list_size, i;
 
-  if (!real_getaddrinfo)
-    real_getaddrinfo = (int (*)(const char *,
-				const char *,
-				const struct addrinfo *,
-				struct addrinfo **))
-      dlsym(RTLD_NEXT, "getaddrinfo");
-
   if (!node || (node && !*node) ||
       !service || (service && !*service) || !hints || !_res)
-    return real_getaddrinfo(node, service, hints, _res);
+    return call_getaddrinfo(node, service, hints, _res);
 
   /* Can't SRV records for numeric hosts. */
   if (hints->ai_flags & AI_NUMERICHOST)
-    return real_getaddrinfo(node, service, hints, _res);
+    return call_getaddrinfo(node, service, hints, _res);
 	
   /* No numeric services either. */
   if (atoi(service) > 0 || *service == '0')
-    return real_getaddrinfo(node, service, hints, _res);
+    return call_getaddrinfo(node, service, hints, _res);
 	
   if (hints->ai_socktype == SOCK_STREAM)
     asprintf(&full_service, "_%s._tcp", service);
   else if (hints->ai_socktype == SOCK_DGRAM)
     asprintf(&full_service, "_%s._udp", service);
   else
-    return real_getaddrinfo(node, service, hints, _res);
+    return call_getaddrinfo(node, service, hints, _res);
 	
   query = ruli_sync_query(full_service, node, -1, RULI_RES_OPT_SEARCH);
   free(full_service);
   if (!query)
-    return real_getaddrinfo(node, service, hints, _res);
+    return call_getaddrinfo(node, service, hints, _res);
 
   srv_code = ruli_sync_srv_code(query);
   if (srv_code) {
     ruli_sync_delete(query);
-    return real_getaddrinfo(node, service, hints, _res);
+    return call_getaddrinfo(node, service, hints, _res);
   }
 
   srv_list = ruli_sync_srv_list(query);
   srv_list_size = ruli_list_size(srv_list);
   if (srv_list_size < 1) {
     ruli_sync_delete(query);
-    return real_getaddrinfo(node, service, hints, _res);
+    return call_getaddrinfo(node, service, hints, _res);
   }
 
   /* scan srv list */
@@ -135,13 +170,15 @@ int ruli_getaddrinfo(const char *node, const char *service,
 	break;
       default:
 	ruli_sync_delete(query);
-	if (res0)
+	if (res0) {
 	  freeaddrinfo(res0);
+	  res0 = 0;
+	}
 	return EAI_FAMILY;
       }
       
       {
-	const int SZ = sizeof(struct addrinfo) + sockaddr_size + canonlen + 1;
+	const int SZ = sizeof(_ruli_ai_addrbag) + canonlen + 1;
 	res1 = ruli_malloc(SZ);
 	if (!res1) {
 	  ruli_sync_delete(query);
@@ -158,17 +195,29 @@ int ruli_getaddrinfo(const char *node, const char *service,
 
       res->ai_family = ruli_addr_family(addr);
       res->ai_socktype = hints->ai_socktype;
-      if (hints->ai_socktype == SOCK_STREAM)
+
+      switch (hints->ai_socktype) {
+      case SOCK_STREAM:
 	res->ai_protocol = IPPROTO_TCP;
-      else if (hints->ai_socktype == SOCK_DGRAM)
+	break;
+      case SOCK_DGRAM:
 	res->ai_protocol = IPPROTO_UDP;
+	break;
+      default:
+	ruli_sync_delete(query);
+	if (res0) {
+	  freeaddrinfo(res0);
+	  res0 = 0;
+	}
+	return EAI_SOCKTYPE;
+      }
 
       switch (ruli_addr_family(addr)) {
       case PF_INET: 
 	{
 	  struct sockaddr_in *ai_addr;
 
-	  ai_addr = (struct sockaddr_in *) (((char *) res) + sizeof(struct addrinfo));
+	  ai_addr = &((_ruli_ai_addrbag *) res)->sa.sa_inet;
 	  ai_addr->sin_port = htons(entry->port);
 	  ai_addr->sin_addr = ruli_addr_inet(addr);
 	  ai_addr->sin_family = ruli_addr_family(addr);
@@ -180,7 +229,7 @@ int ruli_getaddrinfo(const char *node, const char *service,
 	{
 	  struct sockaddr_in6 *ai_addr;
 
-	  ai_addr = (struct sockaddr_in6 *) (((char *) res) + sizeof(struct addrinfo));
+	  ai_addr = &((_ruli_ai_addrbag *) res)->sa.sa_inet6;
 	  ai_addr->sin6_port = htons(entry->port);
 	  ai_addr->sin6_addr = ruli_addr_inet6(addr);
 	  ai_addr->sin6_family = ruli_addr_family(addr);
@@ -190,17 +239,17 @@ int ruli_getaddrinfo(const char *node, const char *service,
 	break;
       default:
 	ruli_sync_delete(query);
-	if (res0)
+	if (res0) {
 	  freeaddrinfo(res0);
+	  res0 = 0;
+	}
 	return EAI_FAMILY;
       }
 
       res->ai_addrlen = sockaddr_size;
 
       if (hints->ai_flags & AI_CANONNAME) {
-	res->ai_canonname = ((char *) res) + 
-	  sizeof(struct addrinfo) +
-	  sockaddr_size;
+	res->ai_canonname = (char *) (((_ruli_ai_addrbag *) res) + 1);
 	memcpy(res->ai_canonname, canonname, canonlen + 1);
       }
 
@@ -212,15 +261,26 @@ int ruli_getaddrinfo(const char *node, const char *service,
 
   /* Something went wrong and res0 isn't populated. */
   if (!res0)
-    return real_getaddrinfo(node, service, hints, _res);
+    return call_getaddrinfo(node, service, hints, _res);
 
 #if 0
   /* Get the normal getaddrinfo() results if possible and put them at
      the end of the linked list.*/
-  if (!real_getaddrinfo(node, service, hints, &res1))
+  if (!call_getaddrinfo(node, service, hints, &res1))
     res->ai_next = res1;
 #endif
 
   *_res = res0;
   return 0;
 }
+
+void ruli_freeaddrinfo(struct addrinfo *res)
+{
+  struct addrinfo *prev;
+
+  while (res) {
+    prev = res;
+    res = res->ai_next;
+    ruli_free(prev);
+  }
+}
