@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_smtp.c,v 1.7 2004/05/24 02:43:21 evertonm Exp $
+  $Id: ruli_smtp.c,v 1.8 2004/06/06 04:27:48 evertonm Exp $
   */
 
 
@@ -224,7 +224,7 @@ static void *on_mx_answer(ruli_res_query_t *qry, void *qry_arg)
 
 	    if (ad_rr->qclass != RULI_RR_CLASS_IN)
 	      continue;
-	    if (ad_rr->type != RULI_RR_TYPE_A)
+	    if (!ruli_rr_type_is_address(srv_qry->srv_options, ad_rr->type))
 	      continue;
 
 	    /*
@@ -294,13 +294,7 @@ static void *on_mx_answer(ruli_res_query_t *qry, void *qry_arg)
 	    }
 	    
 	    /* Write address into space */
-	    {
-	      int result = ruli_parse_rr_a(&addr->addr.ipv4, 
-					   ad_rr->rdata, ad_rr->rdlength);
-	      assert(!result); /* IN A parsing can't fail */
-	    }
-	    addr->addr_family = PF_INET;
-	    addr->addr_size = sizeof(addr->addr.ipv4);
+	    ruli_parse_addr_rr(addr, ad_rr, srv_qry->srv_options);
 
 	  } /* Save MX target addresses */
 	  
