@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_srv.c,v 1.57 2004/05/24 02:43:21 evertonm Exp $
+  $Id: ruli_srv.c,v 1.58 2004/06/06 04:27:48 evertonm Exp $
   */
 
 
@@ -199,13 +199,13 @@ static void *on_walk_answer(ruli_res_query_t *qry, void *qry_arg)
 
     /*
      * As RFC2782 states alias aren't valid SRV targets,
-     * we only consider IN A records in the answer.
+     * we only consider address records in the answer.
      * If alias were permitted, we would need to
      * consider IN CNAME RRs.
      */
 
     /*
-     * Search for IN A records in answer
+     * Search for address records in answer
      */
     {
       ruli_list_t *an_list     = &parse.answer_list;
@@ -217,7 +217,7 @@ static void *on_walk_answer(ruli_res_query_t *qry, void *qry_arg)
 
 	if (rr->qclass != RULI_RR_CLASS_IN)
 	  continue;
-	if (rr->type != RULI_RR_TYPE_A)
+	if (!ruli_rr_type_is_address(srv_qry->srv_options, rr->type))
 	  continue;
 
 	/*
@@ -252,12 +252,7 @@ static void *on_walk_answer(ruli_res_query_t *qry, void *qry_arg)
 	  }
 
 	  /* Write address into space */
-	  {
-	    int result = ruli_parse_rr_a(&addr->addr.ipv4, rr->rdata, rr->rdlength);
-	    assert(!result); /* IN A parsing can't fail */
-	  }
-	  addr->addr_family = PF_INET;
-	  addr->addr_size = sizeof(addr->addr.ipv4);
+	  ruli_parse_addr_rr(addr, rr, srv_qry->srv_options);
 
 	} /* save address */
 
@@ -288,7 +283,7 @@ static void *on_walk_answer(ruli_res_query_t *qry, void *qry_arg)
 	
 	fprintf(stderr, 
 		"DEBUG: on_walk_answer(): query target=%s target_len=%d: "
-		"BUT: no matching IN A record\n", 
+		"BUT: no matching address record\n", 
 		target_txt, target_txt_len);
       }
 #endif
@@ -364,8 +359,10 @@ static void *srv_answer_walk(ruli_srv_t *srv_qry)
     srv_qry->walk_query->q_domain        = entry->target;
     srv_qry->walk_query->q_domain_len    = entry->target_len;
     srv_qry->walk_query->q_class         = RULI_RR_CLASS_IN;
-    srv_qry->walk_query->q_type          = RULI_RR_TYPE_A;
     srv_qry->walk_query->q_options       = RULI_RES_OPT_VOID;
+
+    /* FIXME: perform 2 queries instead? (IN_A, IN_AAAA) */
+    srv_qry->walk_query->q_type          = RULI_RR_TYPE_ANY;
     
     /*
      * Submit walk query
@@ -484,8 +481,8 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
      * The following algorithm scans the answer section
      * for CNAME records:
      *
-     * 1/3: We look for IN A records for the wanted_owner
-     * 2/3: If at least one IN A record is found, we're done
+     * 1/3: We look for address records for the wanted_owner
+     * 2/3: If at least one address record is found, we're done
      * 3/3: We search IN CNAME records for the wanted_owner
      *      If a CNAME record is found, we make
      *      wanted_owner <- cname_target and go to 1/3
@@ -519,14 +516,14 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
 #endif
 
       /*
-       * 1/3: Scan answer section for IN A records
+       * 1/3: Scan answer section for address records
        */
       for (i = 0; i < an_list_size; ++i) {
 	ruli_rr_t *rr = (ruli_rr_t *) ruli_list_get(an_list, i);
 
 	if (rr->qclass != RULI_RR_CLASS_IN)
 	  continue;
-	if (rr->type != RULI_RR_TYPE_A)
+	if (!ruli_rr_type_is_address(srv_qry->srv_options, rr->type))
 	  continue;
 
 	/*
@@ -598,16 +595,11 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
 	  }
 
 	  /* Write address into space */
-	  {
-	    int result = ruli_parse_rr_a(&addr->addr.ipv4, rr->rdata, rr->rdlength);
-	    assert(!result); /* IN A parsing can't fail */
-	  }
-	  addr->addr_family = PF_INET;
-	  addr->addr_size = sizeof(addr->addr.ipv4);
+	  ruli_parse_addr_rr(addr, rr, srv_qry->srv_options);
 
 	} /* save address */
 
-      } /* for: IN A scan of answer section */
+      } /* for: scan answer section for addresses */
 
       /* 
        * 2/3: Done if one SRV record has been built 
@@ -627,7 +619,7 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
 #ifdef RULI_SRV_DEBUG
       fprintf(stderr, 
 	      "DEBUG: on_fallback_answer(): id=%d answer_code=%d, rcode=%d: "
-	      "BUT: no matching IN A record\n",
+	      "BUT: no matching address record\n",
 	      qry->query_id, qry->answer_code, qry->answer_header.rcode);
 #endif
 
@@ -746,7 +738,7 @@ void *_srv_answer_fallback_addr(ruli_srv_t *srv_qry)
     assert(!result);
 
     fprintf(stderr, 
-	    "DEBUG: srv_answer_fallback(): IN A query: domain=%s domain_len=%d\n",
+	    "DEBUG: srv_answer_fallback(): address query: domain=%s domain_len=%d\n",
 	    txt_dname_buf, txt_dname_len);
   }
 #endif
@@ -769,9 +761,11 @@ void *_srv_answer_fallback_addr(ruli_srv_t *srv_qry)
   srv_qry->fall_query->q_domain        = srv_qry->srv_domain;
   srv_qry->fall_query->q_domain_len    = srv_qry->srv_domain_len;
   srv_qry->fall_query->q_class         = RULI_RR_CLASS_IN;
-  srv_qry->fall_query->q_type          = RULI_RR_TYPE_A;
   srv_qry->fall_query->q_options       = srv_qry->srv_options;
 
+  /* FIXME: perform 2 queries instead? (IN_A, IN_AAAA) */
+  srv_qry->fall_query->q_type          = RULI_RR_TYPE_ANY;
+
   /*
    * Submit fallback query
    */
@@ -1244,7 +1238,7 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
 	  if (rr->qclass != RULI_RR_CLASS_IN)
 	    continue;
 
-	  if (rr->type != RULI_RR_TYPE_A)
+	  if (!ruli_rr_type_is_address(srv_qry->srv_options, rr->type))
 	    continue;
 
 	  /* Compare SRV target against additional record owner */
@@ -1267,12 +1261,7 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
 	  }
 
 	  /* Write address into space */
-	  {
-	    int ret = ruli_parse_rr_a(&addr->addr.ipv4, rr->rdata, rr->rdlength);
-	    assert(!ret); /* IN A parsing can't fail */
-	  }
-	  addr->addr_family = PF_INET;
-	  addr->addr_size = sizeof(addr->addr.ipv4);
+	  ruli_parse_addr_rr(addr, rr, srv_qry->srv_options);
 
 	} /* for */
       }
