@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: srvsolver.c,v 1.24 2004/05/24 02:43:21 evertonm Exp $
+  $Id: srvsolver.c,v 1.25 2004/06/01 18:37:35 evertonm Exp $
  */
 
 
@@ -550,27 +550,23 @@ static void parse_servers(ruli_list_t *server_list, int serverc,
   assert(serverc >= 1);
 
   for (i = 0; i < serverc; ++i) {
-    ruli_addr_t *addr;
-
-    addr = malloc(sizeof(*addr));
+    ruli_addr_t *addr = ruli_addr_parse_new(serverv[i]);
     if (!addr) {
-      fprintf(stderr, "%s: can't allocate memory for address\n", prog_name);
-      exit(1);
-    }
-
-    if (!inet_aton(serverv[i], &addr->addr.ipv4)) {
-      fprintf(stderr, "%s: invalid IP address: %s\n", prog_name, serverv[i]);
+      fprintf(stderr, "%s: can't save address: %s\n", prog_name, serverv[i]);
       free(addr);
       exit(1);
     }
 
+#ifdef SRVSOLVER_DEBUG
+    fprintf(stderr, "%s: saving server: ", prog_name);
+    ruli_addr_print(stderr, addr);
+    fprintf(stderr, "\n");
+#endif
+
     {
       int result = ruli_list_push(server_list, addr);
       assert(!result);
     }
-    addr->addr_family = AF_INET;
-    addr->addr_size = sizeof(addr->addr.ipv4);
-    
   }
 }
 
