@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: srvsolver2.c,v 1.15 2004/05/28 22:17:04 evertonm Exp $
+  $Id: srvsolver2.c,v 1.16 2004/06/01 18:37:46 evertonm Exp $
  */
 
 
@@ -540,8 +540,6 @@ static void parse_servers(ruli_list_t *server_list, int serverc,
       int result = ruli_list_push(server_list, addr);
       assert(!result);
     }
-    addr->addr_family = PF_INET;
-    addr->addr_size = sizeof(addr->addr.ipv4);
 
   }
 }
